/*
* =====================================================================
*  Filename: NumEdit.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

class CNumEdit : public CEdit
{

public:

   /// \brief CNumEdit constructor.
   CNumEdit();

private:

   /// \brief Calls this function when trying insert the data from the Clipboard into the CEdit.
   /// \brief Calls this function when trying insert the data from the Clipboard into the CEdit.
   /// \param wParam - specifies additional information about the message
   /// \param lParam - specifies additional information about the message
   /// \return Signed result of message processing.
   afx_msg LRESULT OnPaste(
               WPARAM wParam,
               LPARAM lParam);

   /// \brief The framework calls this member function when a keystroke translates to a nonsystem character.
   /// \param nChar - contains the character code value of the key
   /// \param nRepCnt - contains the repeat count
   /// \param nFlags - contains the scan code, key-transition code, previous key state, and context code
   afx_msg void OnChar(
               UINT nChar,
               UINT nRepCnt,
               UINT nFlags);

   /// \brief The framework calls this member function as a user rotates the mouse wheel 
   /// \brief and encounters the wheel's next notch.
   /// \param nFlags - indicates whether various virtual keys are down
   /// \param zDelta - indicates distance rotated
   /// \param pt - specifies the x- and y-coordinate of the cursor
   /// \return Nonzero if mouse wheel scrolling is enabled; otherwise 0.
   afx_msg BOOL OnMouseWheel(
               UINT nFlags,
               short zDelta,
               CPoint pt);

   /// \brief The framework calls this member function when the user presses the right mouse button.
   /// \param nFlags - indicates whether various virtual keys are down
   /// \param point - Specifies the x and y coordinates of the cursor
   afx_msg void OnRButtonDown(
               UINT nFlags,
               CPoint point);

   /// \brief The framework calls this member function when a child control is about to be drawn.
   /// \param pDC - contains a pointer to the display context for the child window
   /// \param nCtlColor - specifying the type of control
   /// \return Returns a handle to the brush that is to be used for painting the control background
   afx_msg HBRUSH CtlColor(
               CDC* pDC,
               UINT nCtlColor);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief Used by class CWinApp to translate window messages before they are dispatched
   /// \brief to the TranslateMessage and DispatchMessage Windows functions.
   /// \param pMSG - points to a MSG structure that contains the message to process
   /// \return Nonzero if the message was translated and should not be dispatched;
   /// \return 0 if the message was not translated and should be dispatched.
   virtual BOOL PreTranslateMessage(
               MSG *pMSG);

public:

   /// \brief Sets value and stores it into edit control.
   /// \param value - value which will be set
   void SetValue(
               long value);

   /// \brief Stores value into edit control and changes m_allowFractionalPart accordingly.
   /// \param value - value which will be set
   void SetVariantValue(
               const ATL::CComVariant& value);

   /// \brief Returns value from edit control.
   /// \return Value as long.
   long GetValue() const;

   /// \brief Returns value from edit control.
   /// \return Value as CCOMVariant.
   ATL::CComVariant GetVariantValue() const;

   /// \brief Increases edit box value by specified delta.
   /// \param delta - delta by which value will be increased
   void IncreaseValue(
               int delta);

   /// \brief Increases edit box value by specified delta.
   /// \param delta - delta by which value will be increased
   void IncreaseValue(
               double delta);

   /// \brief Decreases edit box value by specified delta.
   /// \param delta - delta by which value will be decreased
   void DecreaseValue(
               int delta);

   /// \brief Decreases edit box value by specified delta.
   /// \param delta - delta by which value will be decreased
   void DecreaseValue(
               double delta);

   /// \brief Sets whether negative values are allowed.
   /// \param isNegativeAllowed - determines whether value can be negative
   void SetNegativeAllowed(
               bool isNegativeAllowed);

   /// \brief Sets whether fractional part is allowed.
   /// \param isFractionAllowed - determines whether fractional part is allowed
   void SetFractionAllowed(
               bool isFractionAllowed);

   /// \brief Sets maximum length of the integer part.
   /// \param maxIntegerPartLength - Maximum length of the integer part
   void SetMaxIntegerPartLength(
               int maxIntegerPartLength);

   /// \brief Sets maximum length of the fractional part.
   /// \param maxFractionalPartLength - Maximum length of the fractional part
   void SetMaxFractionalPartLength(
               int maxFractionalPartLength);

private:

   /// \brief Takes value from edit control and converts it to long.
   /// \return Edit control text converted to long.
   long GetEditBoxValue() const;

   /// \brief Takes value from edit control and converts it to double.
   /// \return Edit control text converted to double.
   double GetEditBoxDblValue() const;

   /// \brief Stores specified long value into edit control.
   /// \param value - value which will be stored into edit control
   void SetEditBoxValue(
               long value);

   /// \brief Stores specified double value into edit control.
   /// \param value - value which will be stored into edit control
   void SetEditBoxDblValue(
               double value);
   
   /// \brief Checks the number validness after entering digit.
   /// \return True if number is valid, otherwise: false
   bool CheckDigit() const;

   /// \brief Checks the number validness after deleting character by backspace button.
   /// \return True if number is valid, otherwise: false
   bool CheckBack() const;

   /// \brief Checks the number validness after deleting character by delete button.
   /// \return True if number is valid, otherwise: false
   bool CheckDelete() const;

   /// \brief Checks specified long number validness.
   /// \return True if number is valid, otherwise: false
   bool CheckValue(
               long value) const;

   /// \brief Checks specified double number validness.
   /// \return True if number is valid, otherwise: false
   bool CheckDblValue(
               double value) const;

private:

   /// \brief Determines whether current value is negative.
   bool m_isNegative;

   /// \brief Determines last placed text.
   CString m_lastText;

   /// \brief Determines whether negative values are allowed.
   bool m_allowNegativeValues;

   /// \brief Determines whether fractional part is allowed.
   bool m_allowFractionalPart;

   /// \brief Maximum characters count in integer part.
   int m_maxIntegerPartLength;

   /// \brief Maximum characters count in fractional part.
   int m_maxFractionalPartLength;

private:

   /// \brief Character used for negative sign.
   static const TCHAR NEGATION_SYMBOL;

   /// \brief Character used for fractional part.
   static const TCHAR FRACTIONAL_PART_SYMBOL;

   /// \brief Text color when value is negative.
   static const COLORREF NEGATIVE_COLOR;

   /// \brief Text color when value is positive.
   static const COLORREF POSITIVE_COLOR;
};
